<?php
    
##  *    ~~ dPlayer.php  V 4.0  ~~
##  *  Copyright Kirk Siqveland 2022
##  *       Edmonds, WA - USA

##  * Core of - dPlayer -
##  * HTML5 Dynamic-Audio Player with Auto-Playlist
    
##  * dPlayer identifies the playable files, builds a playlist and lets you play all the
##  * files in that folder/directory.  If you organize music from albums into folders this
##  * lets you manage the "Playlist" by just adding/removing/renaming items from the folder.
##
##  * Other Features 
##      - can work with no parameters from the folder to be played.
##      - work from the player directory by passing the path to the directory to play.
##      - Auto-builds playlists based on alpha-sort of files.
##          -- With specific naming practices titles are pared to only the song-name
##          -- Simple rename file with a starting number e.g. 05_-_Peter_Gabriel_-_In_Your_Eyes.flac
##      - Use dPlayer playlist file format (.tags) for album and song information and play order.
##      - Use .m3u and .m3u8 playlists for audio file play order.
##      - TO-DO extract more info from the playlists...
##


session_start();

    ## Set up Basic Variable for the embeded player page:
        $foundPlaylist  = ""              ;   ##  Flag - use of playlist in the host folder.
        $PlayerPath     = ""              ;   ##  Empty Path to the dPlayer components directory.

    ## Get the current URL and directory paths
        getPaths()                           ; ## Fill-in constants like rootURL and CurDirPath
        findMyHome($PlayerPath)              ; ## Fill-in Paths like $PlayerPath

    ##  Test for iframe_guest.js in dplayer path (Used to Resize the iframe as dPlayer changes):
        $testPath = $playerPath."iframe_guest.js";
        $insertScriptPath  = "" ;
        if(is_file($testPath)){
            $insertScriptPath  = "<!-- Used to communicate with page hosting this page as an iFrame -->\n" ;
            $insertScriptPath .= "<script  src=".$testPath."></script>\n" ;
        }


    ## Let's Go:
    ## Assumes we have a folder with flac or wav files and a sub directory for mp3's
    ## 
        $playList  = BuildPlayList()         ;
        $mp3List   = BuildPlayList("mp3/")   ; 
        $pegTally  = 0                       ;

?>

<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8" />
    <meta name="author" content="Kirk Siqveland" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=0" />
    <title>HTML5 Dynamic Audio player with auto-playlist</title>

    <!-- add styles and scripts -- $PlayerPath allows a single folder to have all the other files we need -->
    <link   type="text/css"        href="<?php echo $PlayerPath;?>dPlayer.css" rel="stylesheet" />
    <script type="text/javascript" src ="<?php echo $PlayerPath;?>dPlayer.js"></script>
    <script type="text/javascript" src ="<?php echo $PlayerPath;?>KBSliderX.js"></script>    

</head>
<body>
    <div class="centerbox">
        <div class="debugger hidden"><?php echo $DebugMsg; ?></div>
        <div class="nojsmsg"><br/>
            <h2>The dPlayer Audio Player requires JavaScript, which seems to be unavailable!</h2><br/>
        </div>
        <div class="player hidden">
            <div class="pl"  title="Play List"></div>
            <div class="title"></div>
            <div class="subtitle"></div>
            <div class="album"></div>
            <div class="cover">
                <img id="splash">
            </div>
            <div class="controls">
                <div class="play"  title="Play"></div>
                <div class="pause" title="Pause"></div>
                <div class="prev"  title="Previous"></div>
                <div class="rew"   title="Restart"></div>
                <div class="fwd"   title="Next"></div>
            </div>
            <div class="toggles">          
                <div class="mp3on"  title="MP3 - Off"></div> 
                <div class="mp3off" title="MP3 - On"></div>
                <div class="wavon"  title="Flac - Off"></div>  
                <div class="wavoff" title="Flac - On"></div>
            </div>
            <div class="KBSlider" id="volume"  title="Volume"></div>
            <div class="KBSlider" id="songbar" title="Progress"></div>
        </div>

        <ul class="wavlist hidden">
            <?php echo $playList; ?>
        </ul>
        <ul class="mp3list hidden">
            <?php echo $mp3List; ?>
        </ul>        
    </div>

    <?php echo $insertScriptPath;?>
</body>
</html>

<?php    

## ==============================================================================
##                  S u b - R o u t i n e s
## ==============================================================================

function BuildPlayList($subDir = ""){    
    global $CoverArt, $playPath, $FlagVarious, $DebugMsg, $pegTally,  $Performer, $AlbumName ;
    //$localPath = server path to this file (may be source may be play)
    //$playPath   = relative path to the directory with audio files
    $pegTally   = 0                             ;
    $playPath   = $_GET["audiopath"]            ;
    $localRoot  = $_SERVER['DOCUMENT_ROOT']     ;
        // $serverName  = $_SERVER['SERVER_NAME']            ;
        // $CurDirPath  = dirname($_SERVER['PHP_SELF'])      ;
    
    if (empty($playPath)){
        $playPath   = "."  ;
        $localPath  = str_replace($_SERVER['DOCUMENT_ROOT'], "" , __DIR__).'/' ; // Local Path
        $playPath   = CurDirPath.'/'     ; // - as URL
    }else{ 
            if(substr($playPath,0,1)!=="/"){ $playPath = '/' . $playPath        ; }
            if(substr($playPath,-1)!=="/"){ $playPath .= '/'                    ; }
        // Strip $playPath back to a local relative path
        $localPath  = str_replace($_SERVER['DOCUMENT_ROOT'], "" , $playPath)    ;   
        $playPath   = rootURL.$localPath.$subDir ;
    }       

    ## - Scan the current directory for audio files.
    ## Open the current directory | Read Line-BY-Line into $dirArray | Close directory

        $tryPath = $localRoot.$localPath.$subDir ;

        if (is_dir($tryPath)){
            if ($dh = opendir($tryPath)){
                // $DebugMsg .= "Success opening dir: $tryPath  <br>" ;
                // if($asMP3List){ $playPath   .= "mp3/" ; }
                while (($file = readdir($dh)) !== false){
                    if(is_file($tryPath.$file)){
                        // $DebugMsg .= "Test-Stub -- filename: -" . $file . "- <br>" ;
                        $dirArray[]=$file ;
                    }
                    clearstatcache() ;
                }
                closedir($dh);
            }
        }    

    ## Count elements in array
        $indexCount = Count($dirArray) ;
        // $DebugMsg .= "indexCount - file in dir: $indexCount  <br>" ;        
    ## EXIT on EMPTY
        if($indexCount == 0){ 
            $DebugMsg .= "No files in dirArray [Trypath] : $tryPath<br/>" ;
            return "";
        }else{ $DebugMsg .= "$indexCount Files found in dirArray [Trypath] : $tryPath<br/>";}

    ## Now loop through the array of files to find our Artist/Album/Art-tags file
    ## This assumes Named like:
    ##       Artist_-_AlbumName_-_CoverArtFileName.tags
    ## e.g.  Pink_Floyd_-_Dark_Side_of_the_Moon_-_AlbumArt.jpg.tags
    ## This file can be empty, we only use the name.
        // $DebugMsg .= "indexCount = $indexCount <br/>" ;
        for($tick=0; $tick < $indexCount; $tick++) 
        {
            $fileName  =  $dirArray[$tick] ;

            ##  Stub I'm not using but left in in case someone needs it...
            //  $testname = strtolower($fileName) ;
            //  if($testname == "albumart.jpg"||$testname == "albumart.png"||$testname == "coverart.jpg"||$testname == "coverart.png"){ 
            //      $albumArt = $fileName ; 
            // }

            // $DebugMsg .= "Tick dirArray $tick - file: $fileName <br/>"  ;
            $extn=pathinfo($fileName, PATHINFO_EXTENSION) ;

            if($extn == "m3u"||$extn == "m3u8"){
            ## - TODO - Extract more information from m3u files
                ## Assumes music in folders organized by - Performer/Album-or-Performance/All-Songs-and-Audio -
                        $Performer = dirname(__DIR__, 2)    ;
                        $AlbumName = dirname(__DIR__, 1)    ;
                        $foundPlaylist = $dirArray[$tick]   ;
                        $DebugMsg .= "Found Playlist - $fileName <br/>"  ;
            }

            if($extn == "tags"){            
                    $NamePart   = explode ( "_-_" , $fileName )              ;
                        $Performer  = tidyName( $NamePart[0] )               ;
                        $AlbumName  = tidyName( $NamePart[1] )               ;
                            $Cover  = str_replace(".tags","", $NamePart[2])  ;
                        $AlbumArt   = $localPath.$Cover                      ;
                        $CoverArt   = $localPath.$Cover                      ;
                        $foundPlaylist = $dirArray[$tick]                    ;
                        $DebugMsg .= "Found Playlist - $fileName <br/>"      ;
            }
        } /*/ indexCount Loop /*/

$DebugMsg .= "Tags/mp3 File: $foundPlaylist <br/>"  ;

        if(!empty($foundPlaylist)){

                $lPerformer = strtolower($Performer) ;
                    if (strpos($lPerformer,"various")>0 || $lPerformer == "mixtape" || $lPerformer == "mix tape" || $lPerformer == "mix-tape" || $lPerformer == "mix_tape")
                    {   $Performer = ""                     ;
                        $FlagVarious   = true               ;
                    }else { 
                        $FlagVarious   = false              ;
                    }                 

                        $playListPath = $tryPath.$foundPlaylist  ;
                    $fn = fopen($playListPath,"r")          ;
                    if($fn == false){
                        $DebugMsg .= "fOpen Playlist failed! $playListPath<br/>" ;
                    }else{

                        $firstLine = fgets($fn)         ;

                        ## The first line should start with Playlist: 
                        if(strpos($firstLine, 'Playlist:') !== false )
                        {
                            // $foundPlaylist = true           ;
                            // $foundPlaylist = $foundPlaylist;

                            while(! feof($fn))  {
                                ## Get next line in the file

                                    $fullLine  = explode('|',fgets($fn)) ;
                                    $firstPart = trim($fullLine[0]) ;

                                ## Respond according to line type
                                if(!empty($firstPart))
                                {
                                    if(substr($firstPart, 0, 1)=='#')
                                    {
                                            if(substr($firstPart, 1, 1)=='@')
                                            {
                                                $metaInfo = explode('=',$firstPart);
                                                $mTag = strtolower(trim($metaInfo[0]))   ;
                                                switch($mTag)
                                                {
                                                    case "@performer":
                                                        $XPerformer = trim($metaInfo[1]) ;
                                                        break;

                                                    case "@composer":
                                                        $XComposer = trim($metaInfo[1])  ;
                                                        break;

                                                    case "@album":
                                                        $XAlbum = trim($metaInfo[1])     ;
                                                        break;     

                                                    case "@subtitle":
                                                        $XSubtitle = trim($metaInfo[1])  ;
                                                        break;

                                                    case "@albumart":
                                                        $XAlbumArt = trim($metaInfo[1])  ;
                                                        break;

                                                    case "@date":
                                                        $XDate = trim($metaInfo[1])      ;
                                                        break;
                                                }
                                            }
                                    }elseif (substr($firstPart, 0, 1)=='|') 
                                    {   // Ignore insert-line lines      
                                    }elseif (substr($firstPart, 0, 1)=='$') 
                                    {   // Ignore insert-heading lines
                                    }elseif (substr($firstPart, 0, 1)=='!') 
                                    {   // $titleTally += 1                  ;
                                        // $PlayTitles[substr($firstPart, 1)] ;                                   
                                    }else
                                    {   @$extn=pathinfo($firstPart, PATHINFO_EXTENSION) ;   // Get file extension
                                        if ($extn == "flac"||$extn == "mp3"||$extn == "ogg"||$extn == "oga"||$extn == "wav")
                                        {
                                            $fHandle   = fopen($playPath.$firstPart,"r") ;
                                                if($fHandle  !== false)
                                                {              
                                                    if(count($fullLine)>1){ 
                                                        $CoverArt = $localPath.trim($fullLine[1]) ; 
                                                        // $DebugMsg .= " Extra CoverArt: $firstPart == $CoverArt"  ;
                                                    }else{
                                                        $CoverArt = $AlbumArt ;
                                                    }

                                                    $playList .= li_Details($firstPart, strlen($subDir)>0)  ; 
                                                    $pegTally++                                     ;
                                                    // $DebugMsg .= " Added from PlayList: $firstPart"  ;
                                                    // $DebugMsg .= " -:- Performer = $Performer  :: " ; 
                                                    // $DebugMsg .= " AlbumName = $AlbumName<br/>"     ;                                                     
                                                }else{
                                                    $DebugMsg .= "Unable to find $playPath$firstPart<br/>"  ;
                                                }
                                            fclose($fHandle) ;                                                    
                                        }  
                                    }                                      
                                } #/ if(!empty($firstPart))
                            } #/    while(! feof($fn))
                        }else{ 
                            $DebugMsg .= "Empty Playlist <br/>" ;
                            $foundPlaylist = "" ;
                        } #/    if(strpos($firstLine, 'Playlist:') !== false )

                        fclose($fn);
                    }
        } #/ endif !foundplaylist

        if(empty($foundPlaylist)||empty($playList)){
            
            ## Sort file list array
                sort($dirArray) ; 

            ## Loop through the array of files to build a playlist
                for($tick=0; $tick < Count($dirArray) ; $tick++) 
                {                        
                    $playName= $dirArray[$tick] ;  // Gets File Names
                    ## Find Audio files by ending, and add them to the list
                        $extn = pathinfo($playName, PATHINFO_EXTENSION) ; // Get file extension
                        $extn = strtolower($extn) ;

                        if ($extn == "flac"||$extn == "mp3"||$extn == "ogg"||$extn == "oga"||$extn == "wav"){
                            $playList .= li_Details($playName, strlen($subDir)>0)  ;
                            $pegTally++                                     ;
                            // $DebugMsg .= "Added $playName"  ;
                            // $DebugMsg .= " - From folder Performer = $Performer  :: "      ; 
                            // $DebugMsg .= " AlbumName = $AlbumName<br/>"      ;  
                        }
                }#/ end For Loop
        }        

        if(empty($playList))
        {       return ""           ;
        }else{  return $playList    ;
        }        
}



## - List Item Information Extraction Routines -
## This assumes audio files are named using "_-_" as a delimiter between tags
## For dPlayer to work audio-file names must be in this order:

## Notice the Track # is used to help sort files of an album by name and retain
## propper play order.

## (if used)  - Artist name
## (if used)  - Track # -or- Artist name -or- Album name if MixTape or Artist first
## (required) - Track # -or- Artist name if MixTape
## (required) - Song Title
## e.g.   Pink_Floyd_-_Dark_Side_of_The_Moon_-_09_-_Eclipse.flac
## or       09_-_Eclipse.flac
## For Various Artist / MixTape Albums use 03_-_Pink_Floyd_-_Eclipse.flac

    function splitMediaName($MediaName){
        global $Artist, $AlbumName, $Performer, $SongTitle, $FlagVarious, $XPerformer, $XAlbum ;

        if (strpos($MediaName, '_-_') !== false) 
        {
            $NamePart       = explode ( "_-_" , $MediaName ) ;
            $partsCount     = count($NamePart)               ;
            $XSong          = ""                             ;
            switch($partsCount){
                case 2:
                    if (is_numeric($NamePart[0])){
                        $Song       = tidyName( $NamePart[1] , $XSong   ) ;
                    }else{
                        $Artist     = tidyName( $NamePart[0] , $XArtist ) ; 
                        $Song       = tidyName( $NamePart[1] , $XSong   ) ;
                    }   
                    break ;

                case 3:
                    if(preg_match("/^[A|B][0-9]*/", $NamePart[0]) || $FlagVarious  == true ){
                        //$AlbumName   = "MixTape" ;
                        $FlagVarious = true                                 ;
                        $Artist      = tidyName( $NamePart[1] , $XArtist )  ;
                        $Song        = tidyName( $NamePart[2] , $XSong   )  ;
                    }else{                       
                        $Artist      = tidyName( $NamePart[0] , $XArtist )  ;
                        $AlbumName   = tidyName( $NamePart[1] , $XAlbum  )  ;
                        $Song        = tidyName( $NamePart[2] , $XSong   )  ;
                    }
                    break ;

                case 4:
                        $Artist      = tidyName( $NamePart[0] , $XArtist )  ;
                        $AlbumName   = tidyName( $NamePart[1] , $XAlbum  )  ;
                        $Song        = tidyName( $NamePart[3] , $XSong   )  ;
                    break;                    

                default:
                    $Song            = tidyName( $MediaName   , $XSong   )  ;
                    break ;

            } //switch($partsCount)
        }else{
            $Song = tidyName( $MediaName , $XSong   )  ;
        }
                       
        return $Song ;  // function splitMediaName()
    }   

    function tidyName($rawName, $default = ""){
        if(!empty($default)){                $rawName = $default ; }
        $outName = str_replace('_--_',' - ', $rawName)      ;  

        $outName = str_replace('-_', chr(39).' ', $outName) ;  ## Apostrophe - Space
        $outName = str_replace('_-', chr(39),     $outName) ;  ## Apostrophe  &apos;
        // $outName = str_replace('-_', '&apos; ',   $outName) ;  ## Apostrophe - Space
        // $outName = str_replace('_-', '&apos;',    $outName) ;  ## Apostrophe          
        $outName = str_replace('._.', chr(41).'.',$outName) ;  ## Close Paren - Dot
        $outName = str_replace('_.', chr(40),     $outName) ;  ## Open Paren - (        
        $outName = str_replace('._', chr(41),     $outName) ;  ## Close Paren
        $outName = str_replace('.._', ", ", $outName)       ;  ## Comma - Space
        $outName = str_replace('_', " ", $outName)          ;  ## Space        
        $outName = htmlspecialchars($outName)               ;  ## SECURITY CODE
        $outName = str_replace('%20', ' ', $outName)        ;  ## Space
        return $outName ;
    }

    function li_Details($name, $useSubDir = false){
        global $Artist, $AlbumName, $SongTitle, $CoverArt, $Performer, $FlagVarious, $playPath, $pegTally, $DebugMsg ;
            if (empty($CoverArt)){
                $CoverArt = $playPath."CoverThumb.jpg" ;
                if(is_file($CoverArt)===false){$CoverArt = $PlayerPath."CoverThumb.jpg" ;}
            }

            $SMN       = splitMediaName($name);
            $SongName  = str_replace(array(".flac", ".mp3", ".ogg", ".oga", ".wav"),"",$SMN) ;
            $SongTitle = str_replace(chr(39), "&apos;", $SongName);

            if($useSubDir){ $listClass = "mp3Item"  ; 
            }else{          $listClass = "wavItem"  ; }

            if($FlagVarious){
                return "<li class='$listClass' peg='$pegTally' audiourl='$playPath$name' title='$name' displayname='$SongTitle' subtitle='$Artist' cover='$CoverArt' album='$AlbumName'>$SongName<br/> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<small> - $Artist</small></li>"; 
            }else{
                return "<li class='$listClass' peg='$pegTally' audiourl='$playPath$name' title='$name' displayname='$SongTitle' subtitle='$Performer' cover='$CoverArt' album='$AlbumName' >$SongName</li>";
            }     
    }  

    function findMyHome(&$PlayerPath){
        ## Here we look first up two, then up one directory for a folder named "player" to contain all the player files        
        ## - Assuming we are in a specific media folder such as...
        ##      www.mydemodomain.org/wikiroot/media/Pink_Floyd/Metal
        ## - e.g. www.mydemodomain.org/wikiroot/media/player
        ## Barring that we see if a player.dir file was saved to the web-root "www.mydemodomain.org/"
        ## And if all else fails we look in the folder we are already in...

            $lPath = str_replace(CurDirPath, "", __DIR__)               ;
            $upTwo = dirname(__DIR__, 2)                                ;
            $upOne = dirname(__DIR__, 1)                                ;
        if(file_exists("$upTwo/dplayer/dPlayer.css")){
            $PlayerPath = "$upTwo/dplayer/"                              ;
            $PlayerPath = str_replace($lPath, "", $PlayerPath)          ;            
        }else if(file_exists("$upOne/dplayer/dPlayer.css")){
            $PlayerPath = "$upOne/dplayer/"                              ;
            $PlayerPath = str_replace($lPath, "", $PlayerPath)          ;
        }else{
            if(file_exists(rootURL."/player.dir")){
                $PlayerPath = file_get_contents(rootURL."/player.dir")  ;                
            }else{
                if(file_exists("dPlayer.css")){
                    $PlayerPath = __DIR__ . '/'                         ;
                    $PlayerPath = str_replace($lPath, "", $PlayerPath)  ;
                }else{ $PlayerPath = "Couldn't find a Path!"            ; 
                }
            }
        }
    } #/ findMyHome()

    function getPaths(){
    // I'm sure there is a cleaner way to do this but other things I have tried haven't worked.
        if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on'){     
            if (!defined('rootURL')){  define('rootURL', 'https://'.$_SERVER['HTTP_HOST']) ;   }
        }else{      
            if (!defined('rootURL')){  define('rootURL', 'http://'. $_SERVER['HTTP_HOST']) ;   }
        }
        if (!defined('CurDirPath')){   define('CurDirPath', dirname($_SERVER['PHP_SELF'])) ;   }        
    }  

?>